@@ -20,8 +20,9 @@ Shows their status graphically.
 """
 
 __author__ = 'Scott Kirkwood (scott+keymon@forusers.com)'
-__version__ = '1.8'
+__version__ = '1.13'
 
+import locale
 import logging
 import pygtk
 pygtk.require('2.0')
@@ -71,6 +72,15 @@ def fix_svg_key_closure(fname, from_tos):
   return fix_svg_key
 
 
+def cstrf(func):
+  """Change locale before using str function"""
+  OLD_CTYPE = locale.getlocale(locale.LC_CTYPE)
+  locale.setlocale(locale.LC_CTYPE, 'C')
+  s = func()
+  locale.setlocale(locale.LC_CTYPE, OLD_CTYPE)
+  return s
+
+
 class KeyMon:
   """main KeyMon window class."""
 
@@ -107,10 +117,11 @@ class KeyMon:
     self.shape_mask_current = None
     self.shape_mask_cache = {}
 
-    self.MODS = [u'SHIFT', u'CTRL', u'META', u'ALT']
-    self.IMAGES = [u'MOUSE'] + self.MODS
+    self.MODS = ['SHIFT', 'CTRL', 'META', 'ALT']
+    self.IMAGES = ['MOUSE'] + self.MODS
     self.images = dict([(img, None) for img in self.IMAGES])
-    self.enabled = dict((img, self.get_option(img.lower())) for img in self.IMAGES)
+    self.enabled = dict([(img, self.get_option(cstrf(img.lower))) for img in self.IMAGES])
+
 
     self.options.kbd_files = settings.get_kbd_files()
     self.modmap = mod_mapper.safely_read_mod_map(self.options.kbd_file, self.options.kbd_files)
@@ -179,6 +190,8 @@ class KeyMon:
       'SCROLL_UP': [self.svg_name('mouse'), self.svg_name('scroll-up-mouse')],
       'SCROLL_DOWN': [self.svg_name('mouse'), self.svg_name('scroll-dn-mouse')],
 
+      'REL_LEFT': [self.svg_name('mouse'), self.svg_name('sroll-lft-mouse')],
+      'REL_RIGHT': [self.svg_name('mouse'), self.svg_name('scroll-rgt-mouse')],
       'SHIFT': [self.svg_name('shift')],
       'SHIFT_EMPTY': [self.svg_name('shift'), self.svg_name('whiteout-72')],
       'CTRL': [self.svg_name('ctrl')],
@@ -395,7 +408,7 @@ class KeyMon:
     fullname = os.path.join(themepath, '%s%s.svg' % (fname, self.svg_size))
     if self.svg_size and not os.path.exists(fullname):
       # Small not found, defaulting to large size
-      fullname = os.path.join(themepath, '%s/%s.svg' % fname)
+      fullname = os.path.join(themepath, '%s.svg' % fname)
     return fullname
 
   def add_events(self):
@@ -499,6 +512,8 @@ class KeyMon:
         self.reset_no_press_timer()
     elif event.type.startswith('EV_REL') and event.code == 'REL_WHEEL':
       self.handle_mouse_scroll(event.value, event.value)
+    elif event.code.startswith('REL'):
+      self.handle_mouse_scroll(event.value, event.value)
 
   def reset_no_press_timer(self):
     """Initialize no_press_timer"""
@@ -629,21 +644,27 @@ class KeyMon:
   def handle_mouse_button(self, code, value):
     """Handle the mouse button event."""
     if self.enabled['MOUSE']:
-      n_image = 0
-      n_code = 0
-      for i, btn in enumerate(self.btns):
-        if btn == code:
-          n_code = i
-        if btn == self.images['MOUSE'].current:
-          n_image = i
-      if self.options.emulate_middle and ((self.images['MOUSE'].current == 'BTN_LEFT'
-          and code == 'BTN_RIGHT') or
-          (self.images['MOUSE'].current == 'BTN_RIGHT' and code == 'BTN_LEFT')):
-        code = 'BTN_MIDDLE'
-      elif value == 0 and n_code != n_image:
-        code = self.btns[n_image - n_code]
-      elif value == 1 and n_image:
-        code = self.btns[n_image | n_code]
+      if code in self.btns:
+        n_image = 0
+        n_code = 0
+        for i, btn in enumerate(self.btns):
+          if btn == code:
+            n_code = i
+          if btn == self.images['MOUSE'].current:
+            n_image = i
+        if self.options.emulate_middle and ((self.images['MOUSE'].current == 'BTN_LEFT'
+            and code == 'BTN_RIGHT') or
+            (self.images['MOUSE'].current == 'BTN_RIGHT' and code == 'BTN_LEFT')):
+          code = 'BTN_MIDDLE'
+        elif value == 0 and n_code != n_image:
+          code = self.btns[n_image - n_code]
+        elif value == 1 and n_image:
+          code = self.btns[n_image | n_code]
+      elif code not in self.name_fnames:
+        btn_num = code.replace('BTN_', '')
+        self.name_fnames[code] = [
+            fix_svg_key_closure(self.svg_name('mouse'),
+                [('>&#8203;', '>' + btn_num)])]
       self._handle_event(self.images['MOUSE'], code, value)
 
     if self.options.visible_click:
@@ -658,7 +679,11 @@ class KeyMon:
     """Handle the mouse scroll button event."""
     if not self.enabled['MOUSE']:
       return
-    if direction > 0:
+    if direction == 'REL_RIGHT':
+      self._handle_event(self.images['MOUSE'], 'REL_RIGHT', 1)
+    elif direction == 'REL_LEFT':
+      self._handle_event(self.images['MOUSE'], 'REL_LEFT', 1)
+    elif direction > 0:
       self._handle_event(self.images['MOUSE'], 'SCROLL_UP', 1)
     elif direction < 0:
       self._handle_event(self.images['MOUSE'], 'SCROLL_DOWN', 1)
@@ -730,7 +755,7 @@ class KeyMon:
   def settings_changed(self, unused_dlg):
     """Event received from the settings dialog."""
     for img in self.IMAGES:
-      self._toggle_a_key(self.images[img], img, self.get_option(img.lower()))
+      self._toggle_a_key(self.images[img], img, self.get_option(cstrf(img.lower)))
     self.create_buttons()
     self.layout_boxes()
     self.mouse_indicator_win.hide()
@@ -760,7 +785,8 @@ class KeyMon:
     self.update_shape_mask(force=True)
 
     # reload keymap
-    self.modmap = mod_mapper.safely_read_mod_map(self.options.kbd_file, self.options.kbd_files)
+    self.modmap = mod_mapper.safely_read_mod_map(
+            self.options.kbd_file, self.options.kbd_files)
 
   def _toggle_a_key(self, image, name, show):
     """Toggle show/hide a key."""
@@ -829,8 +855,7 @@ def create_options():
                   type='bool',
                   help=_('Make the dialog 25% larger than normal.'))
   opts.add_option(opt_short='-m', opt_long='--meta', dest='meta', type='bool',
-                  ini_group='buttons', ini_name='meta',
-                  default=False,
+                  ini_group='buttons', ini_name='meta', default=None,
                   help=_('Show the meta (windows) key.'))
   opts.add_option(opt_long='--mouse', dest='mouse', type='bool', default=True,
                   ini_group='buttons', ini_name='mouse',
@@ -972,7 +997,7 @@ def main():
   opts = create_options()
   opts.read_ini_file(os.path.join(settings.get_config_dir(), 'config'))
   desc = _('Usage: %prog [Options...]')
-  opts.parse_args(desc)
+  opts.parse_args(desc, sys.argv)
 
   if opts.version:
     show_version()
